@@ -25,31 +25,36 @@ use std::fmt;
 use std::rc::Rc;
 use std::vec::Vec;
 
-use ::action::Action;
-use ::drawing;
-use ::float_ord::FloatOrd;
-use ::keyboard::KeyState;
-use ::logging;
-use ::manager;
-use ::submission::{ Submission, SubmitData, Timestamp };
-use ::util::find_max_double;
+use crate::action::Action;
+use crate::actors;
+use crate::drawing;
+use crate::float_ord::FloatOrd;
+use crate::keyboard::KeyState;
+use crate::logging;
+use crate::popover;
+use crate::receiver;
+use crate::submission::{ Submission, SubmitData, Timestamp };
+use crate::util::find_max_double;
 
-use ::imservice::ContentPurpose;
+use crate::imservice::ContentPurpose;
 
 // Traits
 use std::borrow::Borrow;
-use ::logging::Warn;
+use crate::logging::Warn;
 
 /// Gathers stuff defined in C or called by C
 pub mod c {
     use super::*;
-
-    use gtk_sys;
-    use std::os::raw::c_void;
+    
+    use crate::receiver;
     use crate::submission::c::Submission as CSubmission;
 
+    use gtk_sys;
     use std::ops::{ Add, Sub };
-
+    use std::os::raw::c_void;
+    
+    use crate::util::CloneOwned;
+    
     // The following defined in C
     #[repr(transparent)]
     #[derive(Copy, Clone)]
@@ -215,13 +220,17 @@ pub mod c {
             submission: CSubmission,
             widget_to_layout: Transformation,
             time: u32,
-            manager: manager::c::Manager,
+            popover: actors::popover::c::Actor,
+            app_state: receiver::c::State,
             ui_keyboard: EekGtkKeyboard,
         ) {
             let time = Timestamp(time);
             let layout = unsafe { &mut *layout };
             let submission = submission.clone_ref();
             let mut submission = submission.borrow_mut();
+            let app_state = app_state.clone_owned();
+            let popover_state = popover.clone_owned();
+            
             let ui_backend = UIBackend {
                 widget_to_layout,
                 keyboard: ui_keyboard,
@@ -236,7 +245,7 @@ pub mod c {
                     &mut submission,
                     Some(&ui_backend),
                     time,
-                    Some(manager),
+                    Some((&popover_state, app_state.clone())),
                     key,
                 );
             }
@@ -315,13 +324,18 @@ pub mod c {
             x_widget: f64, y_widget: f64,
             widget_to_layout: Transformation,
             time: u32,
-            manager: manager::c::Manager,
+            popover: actors::popover::c::Actor,
+            app_state: receiver::c::State,
             ui_keyboard: EekGtkKeyboard,
         ) {
             let time = Timestamp(time);
             let layout = unsafe { &mut *layout };
             let submission = submission.clone_ref();
             let mut submission = submission.borrow_mut();
+            // We only need to query state here, not update.
+            // A copy is enough.
+            let popover_state = popover.clone_owned();
+            let app_state = app_state.clone_owned();
             let ui_backend = UIBackend {
                 widget_to_layout,
                 keyboard: ui_keyboard,
@@ -352,7 +366,7 @@ pub mod c {
                             &mut submission,
                             Some(&ui_backend),
                             time,
-                            Some(manager),
+                            Some((&popover_state, app_state.clone())),
                             key,
                         );
                     }
@@ -377,7 +391,7 @@ pub mod c {
                         &mut submission,
                         Some(&ui_backend),
                         time,
-                        Some(manager),
+                        Some((&popover_state, app_state.clone())),
                         key,
                     );
                 }
@@ -1035,7 +1049,11 @@ mod seat {
         submission: &mut Submission,
         ui: Option<&UIBackend>,
         time: Timestamp,
-        manager: Option<manager::c::Manager>,
+        // TODO: intermediate measure:
+        // passing state conditionally because it's only used for popover.
+        // Eventually, it should be used for sumitting button events,
+        // and passed always.
+        manager: Option<(&actors::popover::State, receiver::State)>,
         rckey: &Rc<RefCell<KeyState>>,
     ) {
         let key: KeyState = {
@@ -1070,7 +1088,7 @@ mod seat {
             // only show when UI is present
             Action::ShowPreferences => if let Some(ui) = &ui {
                 // only show when layout manager is available
-                if let Some(manager) = manager {
+                if let Some((manager, app_state)) = manager {
                     let view = layout.get_current_view();
                     let places = ::layout::procedures::find_key_places(
                         view, &rckey,
@@ -1085,10 +1103,11 @@ mod seat {
                             width: button.size.width,
                             height: button.size.height,
                         };
-                        ::popover::show(
+                        popover::show(
                             ui.keyboard,
                             ui.widget_to_layout.reverse_bounds(bounds),
                             manager,
+                            app_state,
                         );
                     }
                 }
