@@ -23,6 +23,7 @@ mod assert_matches;
 mod logging;
 
 mod action;
+mod actors;
 mod animation;
 pub mod data;
 mod debug;
@@ -34,10 +35,10 @@ mod keyboard;
 mod layout;
 mod locale;
 mod main;
-mod manager;
 mod outputs;
 mod panel;
 mod popover;
+mod receiver;
 mod resources;
 mod state;
 mod style;
